<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

$billingFields = include_once \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/wc/billingFields.php');

$resources = new \WcFattureGratisInvoice\Resources();
$optionPage = \WcFattureGratisInvoice\Admin\Settings\OptionPage::init();
$invoiceFields = new \WcFattureGratisInvoice\WooCommerce\Fields\InvoiceFields($billingFields, $optionPage);

return apply_filters('wc_fatture_gratis_inv-filters_admin', [
    'admin' => [
        'action' => [
            /*
             * Options
             *
             * - addPluginPage   @since 1.0.0
             * - pageOptionsInit @since 1.0.0
             */
            [
                'filter' => 'admin_menu',
                'callback' => [$optionsPage, 'addPluginPage'],
                'priority' => 10,
            ],
            [
                'filter' => 'admin_init',
                'callback' => [$optionsPage, 'pageOptionsInit'],
                'priority' => 10,
            ],
            // array(
            //     'filter'   => 'admin_init',
            //     'callback' => array($generalShopFields, 'wcGeneralRedirect'),
            //     'priority' => 10,
            // ),

            /*
             * WooCommerce
             *
             * - set invoice number on order completed @since 1.0.0
             * - set invoice number on order refund    @since 1.0.0
             * - set initial invoice number            @since 1.0.0
             * - extra billing fields                  @since 1.0.0
             * - edit invoice meta                     @since 1.0.0
             * - save invoice meta                     @since 1.0.0
             * - remove sent invoice attachment        @since 1.0.0
             */
            [
                'filter' => 'woocommerce_order_status_changed',
                'callback' => 'WcFattureGratisInvoice\\Functions\\setInvoiceNumberOnOrderCompleted',
                'priority' => 20,
                'accepted_args' => 3,
            ],
            [
                'filter' => 'woocommerce_refund_created',
                'callback' => 'WcFattureGratisInvoice\\Functions\\setInvoiceNumberOnOrderRefund',
                'priority' => 20,
            ],
            [
                'filter' => 'init',
                'callback' => 'WcFattureGratisInvoice\\Functions\\setInitInvoiceNumber',
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_admin_order_data_after_billing_address',
                'callback' => [$invoiceFields, 'viewBillingFieldsFilter'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_admin_order_data_after_order_details',
                'callback' => [$invoiceFields, 'editGeneralFieldsFilter'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_admin_order_items_after_refunds',
                'callback' => [$invoiceFields, 'editInvoiceDataOrderRefund'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_admin_order_totals_after_refunded',
                'callback' => [$invoiceFields, 'refundedPaymentMethod'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_process_shop_order_meta',
                'callback' => [$invoiceFields, 'saveOrderMetaBox'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_process_shop_order_meta',
                'callback' => [$invoiceFields, 'saveRefundMetaBox'],
                'priority' => 20,
            ],
            // array(
            //     'filter'   => 'woocommerce_view_order',
            //     'callback' => 'WcFattureGratisInvoice\\Functions\\WebService\\hookActionPaymentConfirmation',
            //     'priority' => 20,
            // ),

            /*
             * Enqueue @since 1.0.0
             */
            [
                'filter' => 'admin_enqueue_scripts',
                'callback' => [$resources, 'register'],
                'priority' => 10,
            ],
            [
                'filter' => 'admin_enqueue_scripts',
                'callback' => [$resources, 'enqueue'],
                'priority' => 20,
            ],
            [
                'filter' => 'admin_enqueue_scripts',
                'callback' => [$resources, 'localizeScript'],
                'priority' => 30,
            ],
            // array(
            //     'filter'   => 'admin_enqueue_scripts',
            //     'callback' => array($resources, 'loadWpMediaFiles'),
            //     'priority' => 40,
            // ),

            /*
             * Ajax mark "sent/no_sent" invoice
             */
            [
                'filter' => 'wp_ajax_markInvoice',
                'callback' => [$invoiceFields, 'markInvoice'],
                'priority' => 10,
            ],
        ],
        'filter' => [
            /*
             * WooCommerce
             *
             * - customer fields                  @since 1.0.0
             * - billing fields                   @since 1.0.0
             * - ajax customer fields             @since 1.0.0
             * - general setting invoice fields   @since 1.0.0
             * - general setting invoice sanitize @since 1.0.0
             * - wc general notice                @since 1.0.0
             */
            [
                'filter' => 'woocommerce_customer_meta_fields',
                'callback' => [$invoiceFields, 'customerFieldsFilter'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_admin_billing_fields',
                'callback' => [$invoiceFields, 'editBillingFieldsFilter'],
                'priority' => 20,
            ],
            [
                'filter' => 'woocommerce_ajax_get_customer_details',
                'callback' => [$invoiceFields, 'foundCustomerMeta'],
                'priority' => 20,
                'accepted_args' => 3,
            ],
            // array(
            //     'filter'   => 'woocommerce_general_settings',
            //     'callback' => array($generalShopFields, 'generalInvoiceFields'),
            //     'priority' => 20,
            // ),
            // array(
            //     'filter'        => 'woocommerce_admin_settings_sanitize_option',
            //     'callback'      => array($generalShopFields, 'sanitize'),
            //     'priority'      => 20,
            //     'accepted_args' => 3,
            // ),
            // array(
            //     'filter'   => 'admin_notices',
            //     'callback' => array($generalShopFields, 'notice'),
            //     'priority' => 20,
            // ),
        ],
    ],
]);
