<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

$fields = include_once \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/wc/billingFields.php');

$endPointApi = new \WcFattureGratisInvoice\EndPoint\Endpoints();
$resources = new \WcFattureGratisInvoice\Resources();
$optionPage = \WcFattureGratisInvoice\Admin\Settings\OptionPage::init();
$invoiceFields = new \WcFattureGratisInvoice\WooCommerce\Fields\InvoiceFields($fields, $optionPage);

return apply_filters('wc_fatture_gratis_inv-filters_front', [
    'front' => [
        'action' => [
            /*
             * WooCommerce
             *
             * - checkout process @since 1.0.0
             * - address process @since 1.0.0
             */
            [
                'filter' => [
                    'woocommerce_checkout_process',
                    'woocommerce_after_save_address_validation',
                ],
                'callback' => [$invoiceFields, 'process'],
                'priority' => 20,
            ],

            /*
             * Enqueue @since 1.0.0
             */
            [
                'filter' => 'wp_enqueue_scripts',
                'callback' => [$resources, 'enqueue'],
                'priority' => 20,
            ],
            [
                'filter' => 'wp_enqueue_scripts',
                'callback' => [$resources, 'localizeScript'],
                'priority' => 30,
            ],
            [
                'filter' => 'wc_fatture_gratis_inv-add_json_header',
                'callback' => function () {
                    /*
                     * The origin parameter specifies a URI that may access the resource.
                     * The browser must enforce this. For requests
                     * without credentials, the server may specify "*" as a wildcard,
                     * thereby allowing any origin to access the resource.
                     */
                    header('Access-Control-Allow-Origin: *');

                    /*
                     * The Access-Control-Allow-Methods header specifies the method or methods allowed when accessing
                     * the resource. This is used in response to a preflight request. The conditions under
                     * which a request is preflighted are discussed above.
                     */
                    header('Access-Control-Allow-Methods: POST, GET');

                    /*
                     * The Access-Control-Allow-Credentials response header tells browsers whether to expose the
                     * response to frontend JavaScript code when the request's
                     * credentials mode (Request.credentials) is "include".
                     */
                    header('Access-Control-Allow-Credentials: true');
                },
                'priority' => 10,
            ],

            /*
             * Add Endpoint @since 1.0.0
             */
            [
                'filter' => 'wp',
                'callback' => [$endPointApi, 'setQueryArgs'],
                'priority' => 10,
            ],
            [
                'filter' => 'init',
                'callback' => [$endPointApi, 'addEndPoint'],
                'priority' => 10,
            ],
            [
                'filter' => 'init',
                'callback' => [$endPointApi, 'addRewriteEndpoint'],
                'priority' => 10,
            ],

            /*
             * - WC auto completed and processing order @since 1.0.0
             */
            [
                'filter' => [
                    'woocommerce_order_status_completed',
                    'woocommerce_order_status_processing',
                ],
                'callback' => 'WcFattureGratisInvoice\\Functions\\setInvoiceNumberOnOrderAutoCompleted',
                'priority' => 10,
                'accepted_args' => 2,
            ],
        ],
        'filter' => [
            /*
             * WooCommerce
             *
             * - my account formatted address      @since 1.0.0
             * - billig fields                     @since 1.0.0
             * - formatted address replacements    @since 1.0.0
             * - localization address replacements @since 1.0.0
             * - my account order action           @since 1.0.0
             */
            [
                'filter' => 'woocommerce_my_account_my_address_formatted_address',
                'callback' => [$invoiceFields, 'myAccountFormattedAddress'],
                'priority' => 20,
                'accepted_args' => 3,
            ],
            [
                'filter' => 'woocommerce_order_formatted_billing_address',
                'callback' => [$invoiceFields, 'orderFormattedBillingAddress'],
                'priority' => 20,
                'accepted_args' => 2,
            ],
            [
                'filter' => 'woocommerce_billing_fields',
                'callback' => [$invoiceFields, 'billingAddressFields'],
                'priority' => 20,
                'accepted_args' => 3,
            ],
            [
                'filter' => 'woocommerce_formatted_address_replacements',
                'callback' => [$invoiceFields, 'formattedAddressReplacements'],
                'priority' => 20,
                'accepted_args' => 2,
            ],
            [
                'filter' => 'woocommerce_my_account_my_orders_actions',
                'callback' => [$invoiceFields, 'actionsFront'],
                'priority' => 20,
                'accepted_args' => 2,
            ],

            /*
             * Add Query Vars @since 1.0.0
             */
            [
                'filter' => 'query_vars',
                'callback' => [$endPointApi, 'addQueryVars'],
                'priority' => 10,
            ],
        ],
    ],
]);
