<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

$page = \WcFattureGratisInvoice\Admin\Settings\OptionPage::init();
$options = $page->getOptions();
$countries = new \WC_Countries();

$country = is_user_logged_in() ?
    get_user_meta(get_current_user_id(), 'billing_country', true) :
    $countries->get_base_country();

$euVatCountry = $countries->get_vat_countries();

$scripts = [];

if (!is_admin()) {
    $scripts = array_merge($scripts, [
        'wc_fatture_gratis_inv_invoice' => [
            'ajax_url' => untrailingslashit(admin_url('admin-ajax.php')),
            'ajax_nonce' => wp_create_nonce('wc_fatture_gratis_inv_ajax-ajax_nonce'),
            'required_text' => esc_html__('required', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'not_required_text' => esc_html__('not required', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'invoice_required' => 'required' === $page->getOptions('invoice_required') ? 1 : 0,
            'hide_outside_ue' => $page->getOptions('hide_outside_ue'),
            'disable_pec_sdi' => $page->getOptions('invoice_disable_pec_sdi'),
            'disable_cf' => $page->getOptions('invoice_disable_cf'),
            'country' => $country ?: '',
            'user_country' => get_user_meta(get_current_user_id(), 'billing_country', true) ?: '',
            'eu_vat_country' => $euVatCountry ?: '',
            'sdi_label' => esc_html__('Certified e-mail (PEC) or the unique code', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'sdi_placeholder' => esc_html__('E-mail (PEC) or the unique code', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'sdi_description' => esc_html__('Please enter your certified e-mail (PEC) or the unique code',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN),
        ],
    ]);
} else {
    $scripts = array_merge($scripts, [
        'wc_fatture_gratis_inv_admin' => [
            'ajax_url' => untrailingslashit(admin_url('admin-ajax.php')),
            'ajax_nonce' => wp_create_nonce('wc_fatture_gratis_inv_ajax-ajax_nonce'),
            'required_text' => esc_html__('required', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'not_required_text' => esc_html__('not required', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'text_edit' => esc_html__('Edit', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'text_save' => esc_html__('Save', WC_FATTUREGRATIS_INV_TEXTDOMAIN),
            'invoice_sent_confirm' => esc_html__(
                'WARNING: You are about to set the "SENT to invoice" status, do you want to confirm?',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ),
            'invoice_undo_confirm' => esc_html__(
                'WARNING: You are about to set the "NOT SENT to invoice" status, do you want to confirm?',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ),
            'bulk_invoice_cb' => esc_html__(
                'First you must select one or more checkbox',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ),
            'refund_item_disabled_text' => esc_html__(
                'Refund disabled: first you need to send the generated credit note',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ),
            'refund_amount_read_only_info_text' => esc_html__(
                'Before sending the invoice you cannot make a refund, on the total but only on the lines of the order',
                WC_FATTUREGRATIS_INV_TEXTDOMAIN
            ),
        ],
    ]);
}

return apply_filters('wc_fatture_gratis_inv-scripts_localize_list', $scripts);
