<?php

/**
 * FattureGratis.
 *
 * Plugin Name: FattureGratis
 * Plugin URI: https://fatturegratis.eu
 * Description: <code><strong>FattureGratis Fattura Elettronica</strong></code>, Questo plugin permette di inviare le fatture tramite il servizio di FattureGratis. Il plugin permette di scaricare, da interfaccia amministratore, sia l'xml che il pdf. C'è anche un file, che racchiude tutte le informazioni relative alla fattura, alla sua elaborazione in formato JSon che si può scaricare dall'interfaccia amministratore. Il plugin aggiunge i campi necessari per la composizione della fattura elettronica all'e-commerce. Passa alla <strong><a href="https://fatturegratis.eu/">PIATTAFORMA</a></strong>
 * Version: 1.0.0
 * Author: FattureGratis <wordpress@fatturegratis.eu>
 * Author URI: https://fatturegratis.eu
 * WC requires at least: 3.2.0
 * WC tested up to: 3.6.2
 * License GPL 2 Text
 * Domain: el-inv
 *
 * Copyright (C) 2021 FattureGratis <wordpress@fatturegratis.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!defined('WC_FATTUREGRATIS_INV_ENV')) {
    define('WC_FATTUREGRATIS_INV_ENV', 'prod');
}

$dirs = [
    '/files/pdf',
    '/files/xml',
    '/files/info',
    '/certs',
];

foreach ($dirs as $dir_name) {
    $dir = __DIR__.$dir_name;
    if (!is_dir($dir)) {
        mkdir($dir, 0755, true);
    }
}

// Define constant.
define('WC_FATTUREGRATIS_INV_NAME', 'FattureGratis');
define('WC_FATTUREGRATIS_INV_TEXTDOMAIN', 'el-inv');
define('WC_FATTUREGRATIS_INV_VERSION', '1.0.0');
define('WC_FATTUREGRATIS_INV_PLUGIN_DIR', basename(plugin_dir_path(__FILE__)));
define('WC_FATTUREGRATIS_INV_DIR', plugin_dir_path(__FILE__));
define('WC_FATTUREGRATIS_INV_URL', plugin_dir_url(__FILE__));
// Base Requirements.
require_once untrailingslashit(WC_FATTUREGRATIS_INV_DIR.'/src/Plugin.php');
require_once untrailingslashit(WC_FATTUREGRATIS_INV_DIR.'/requires.php');
require_once \WcFattureGratisInvoice\Plugin::getPluginDirPath('/src/Autoloader.php');
// Setup Autoloader.
$loaderMap = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/autoloaderMapping.php');
$loader = new \WcFattureGratisInvoice\Autoloader();
$loader->addNamespaces($loaderMap);
$loader->register();
// Register the activation hook.
register_activation_hook(__FILE__, ['WcFattureGratisInvoice\\Activate', 'activate']);
register_deactivation_hook(__FILE__, ['WcFattureGratisInvoice\\Deactivate', 'deactivate']);

use WcFattureGratisInvoice\Functions\WebService;
use WcFattureGratisInvoice\Resources;

// Init
add_action('plugins_loaded', function () {
    function my_mime_types($mime_types)
    {
        $mime_types['p12'] = 'application/x-pkcs12';
        // $mime_types['pem'] = 'application/x-pem-file';
        // $mime_types['crt'] = 'application/x-x509-ca-cert';
        // $mime_types['key'] = 'application/pkcs8';

        // $mime_types['p12'] = 'text/plain';
        $mime_types['pem'] = 'text/plain';
        $mime_types['crt'] = 'text/plain';
        $mime_types['key'] = 'text/plain';

        return $mime_types;
    }
    add_filter('upload_mimes', 'my_mime_types', 1, 1);

    $resources = new Resources();
    add_action('admin_enqueue_scripts', [$resources, 'loadWpMediaFiles']);

    $webService = new WebService();

    add_action('woocommerce_order_status_changed', [$webService, 'hookActionPaymentConfirmation'], 10, 1);
    add_action('woocommerce_admin_order_data_after_order_details', [$webService, 'hookActionPaymentConfirmation'], 10, 1);
    // Load plugin text-domain.
    load_plugin_textdomain('fatture-gratis-inv', false, '/'.WC_FATTUREGRATIS_INV_PLUGIN_DIR.'/languages/');
    // Check for the dependency.
    if (\WcFattureGratisInvoice\Functions\isWooCommerceActive()) {
        $filters = [];
        // Global filter
        $filters = array_merge(
        $filters,
        include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/filters.php')
    );
        $filters = array_merge(
        $filters,
        include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/filters.php')
    );
        // Admin filter
        if (is_admin()) {
            $filters = array_merge(
            $filters,
            include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/filtersAdmin.php')
        );
        } // Front filter
        else {
            $filters = array_merge(
            $filters,
            include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/filtersFront.php')
        );
        }
        // Loader init.
        $init = new WcFattureGratisInvoice\Init(new WcFattureGratisInvoice\Loader(), $filters);
        $init->init();
        // Settings plugin init.
        \WcFattureGratisInvoice\Admin\Settings\OptionPage::init();
    } else {
        // WooCommerce not active, lets disable the plugin.
        \WcFattureGratisInvoice\Functions\disablePlugin();
    }
}, 20);
