<?php

namespace WcFattureGratisInvoice\Admin\Settings\Fields;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Admin\Settings\OptionFields;
use WcFattureGratisInvoice\Admin\Settings\OptionPage;

class Checkbox extends Text
{
    /**
     * Allow Html.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $allowHtml = [];

    /**
     * Checkbox constructor.
     *
     * @since 1.0.0
     *
     * @param $args
     */
    public function __construct($args, OptionFields $fields, OptionPage $page)
    {
        $args = wp_parse_args($args, [
            'value' => 'on',
            'type' => 'checkbox',
        ]);

        $this->allowHtml = [
            'label' => [
                'for' => true,
            ],
            'input' => [
                'class' => true,
                'type' => true,
                'id' => true,
                'name' => true,
                'value' => true,
                'checked' => true,
                'disabled' => true,
                'required' => true,
            ],
            'p' => true,
            'br' => true,
            'small' => true,
            'ul' => true,
            'li' => true,
        ];

        parent::__construct($args, $fields, $page);
    }

    /**
     * Output Field.
     *
     * @since 1.0.0
     */
    public function field()
    {
        $labelBefore = '';
        $labelAfter = '';
        $description = '';

        if (isset($this->args['label'])) {
            $labelBefore = sprintf('<label for="%s">', esc_attr($this->args['id']));
            $labelAfter = sprintf('%s</label>', esc_html__($this->args['label'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
        }

        if (isset($this->args['description'])) {
            $description = sprintf('<br><small>%s</small>',
                esc_html__($this->args['description'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
        }

        $output = sprintf(
            '%1$s <input class="%2$s" type="%3$s" name="%4$s[%5$s]" id="%6$s" value="%7$s" %8$s/>%9$s %10$s',
            $labelBefore,
            esc_attr($this->args['class']),
            sanitize_key($this->args['type']),
            $this->page->optionsName,
            esc_attr($this->args['name']),
            esc_attr($this->args['id']),
            $this->args['value'],
            checked($this->escape(), 'on', false),
            $labelAfter,
            $description
        );

        /**
         * Output Filter.
         *
         * @since 1.0.0
         *
         * @param string $output the output of the input type
         * @param Checkbox The instance class
         */
        $output = apply_filters('wc_fatture_gratis_inv-input_checkbox_output', $output, $this);

        /*
         * Before input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_checkbox_before_input_action', $this->args);

        echo wp_kses($output, $this->allowHtml);

        /*
         * After input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_checkbox_after_input_action', $this->args);
    }
}
