<?php

namespace WcFattureGratisInvoice\Admin\Settings\Fields;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Admin\Settings\OptionFields;
use WcFattureGratisInvoice\Admin\Settings\OptionPage;

class Radio extends Text
{
    /**
     * Allow Html.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $allowHtml = [];

    /**
     * Radio constructor.
     *
     * @since 1.0.0
     *
     * @param $args
     */
    public function __construct($args, OptionFields $fields, OptionPage $page)
    {
        $args = wp_parse_args($args, [
            'options' => [],
            'type' => 'radio',
        ]);

        $this->allowHtml = [
            'label' => [
                'for' => true,
            ],
            'input' => [
                'class' => true,
                'type' => true,
                'id' => true,
                'name' => true,
                'value' => true,
                'checked' => true,
                'disabled' => true,
                'required' => true,
            ],
            'p' => true,
            'br' => true,
            'small' => true,
            'ul' => true,
            'li' => true,
        ];

        parent::__construct($args, $fields, $page);
    }

    /**
     * Escape.
     *
     * @since  1.0.0
     *
     * @param null $value
     *
     * @return string string The escaped value of this type
     */
    public function escape($value = null)
    {
        $value = $value ?: $this->getValue();

        return esc_html($value);
    }

    /**
     * Get Saved Value.
     *
     * @since  1.0.0
     *
     * @return string
     */
    private function getValue()
    {
        $options = $this->fields->options;
        $newValue = isset($options[$this->args['name']]) ? $options[$this->args['name']] : '';

        return esc_html($newValue);
    }

    /**
     * Output Field.
     *
     * @since 1.0.0
     */
    public function field()
    {
        $labelBefore = '';
        $labelAfter = '';
        $description = '';

        if (empty($this->args['options'])) {
            return '';
        }

        $radios = [];
        $c = 1;

        if (isset($this->args['description'])) {
            $description = sprintf('<p><small>%s</small></p>',
                esc_html__($this->args['description'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
        }

        foreach ($this->args['options'] as $value => $label) {
            $labelString = esc_html__($label, WC_FATTUREGRATIS_INV_TEXTDOMAIN);
            if (isset($this->args['label'])) {
                $labelBefore = sprintf('<label for="%s">', esc_attr($this->args['id'].'-'.$c));
                $labelAfter = sprintf('%s</label>', $labelString);
            }

            array_push($radios, sprintf(
                '<li>%1$s <input type="%2$s" name="%3$s[%4$s]" id="%5$s" value="%6$s" %7$s/>%8$s</li>',
                $labelBefore,
                sanitize_key($this->args['type']),
                $this->page->optionsName,
                esc_attr($this->args['name']),
                esc_attr($this->args['id'].'-'.$c),
                $this->escape($value),
                checked($this->escape(), $this->escape($value), false),
                $labelAfter
            ));

            ++$c;

            $output = '<ul>'.implode("\n", $radios).'</ul>'.$description;
        }

        /**
         * Output Filter.
         *
         * @since 1.0.0
         *
         * @param string $output the output of the input type
         * @param Radio The instance class
         */
        $output = apply_filters('wc_fatture_gratis_inv-input_radio_output', $output, $this);

        /*
         * Before input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_radio_before_input_action', $this->args);

        echo wp_kses($output, $this->allowHtml);

        /*
         * After input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_radio_after_input_action', $this->args);
    }
}
