<?php

namespace WcFattureGratisInvoice\Admin\Settings\Fields;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Admin\Settings\OptionFields;
use WcFattureGratisInvoice\Admin\Settings\OptionPage;

class Text
{
    /**
     * Args for fields.
     *
     * @since 1.0.0
     *
     * @var array
     */
    public $args;

    /**
     * OptionFields.
     *
     * @since 1.0.0
     *
     * @var
     */
    public $fields;

    /**
     * Page.
     *
     * @since 1.0.0
     *
     * @var
     */
    public $page;

    /**
     * Allow Html.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $allowHtml = [];

    /**
     * Text constructor.
     *
     * @since 1.0.0
     *
     * @param $args
     */
    public function __construct($args, OptionFields $fields, OptionPage $page)
    {
        $args = wp_parse_args($args, [
            'type' => 'text',
            'name' => 'wc_fatture_gratis_inv-text_field',
            'id' => 'wc_fatture_gratis_inv-text_field',
            'class' => '',
            'label' => '',
            'description' => '',
            'placeholder' => '',
            'filter' => FILTER_SANITIZE_STRING,
            'filter_options' => [
                'flags' => FILTER_FLAG_NO_ENCODE_QUOTES | FILTER_FLAG_ENCODE_AMP,
            ],
            'attrs' => [],
        ]);

        $this->allowHtml = [
            'label' => [
                'for' => true,
            ],
            'input' => [
                'class' => true,
                'placeholder' => true,
                'type' => true,
                'name' => true,
                'id' => true,
                'value' => true,
                'disabled' => true,
                'required' => true,
                // For Input Number.
                'min' => true,
                'max' => true,
            ],
            'p' => true,
            'br' => true,
            'small' => true,
            'ul' => true,
            'li' => true,
        ];

        $this->args = $args;
        $this->fields = $fields;
        $this->page = $page;
    }

    /**
     * Escape.
     *
     * @since  1.0.0
     *
     * @return string The escaped value of this type
     */
    public function escape()
    {
        $value = isset($this->fields->options[$this->args['name']]) ? $this->fields->options[$this->args['name']] : '';

        return esc_html($value);
    }

    /**
     * Get Attrs.
     *
     * @since  1.0.0
     *
     * @param $attrs array The attributes Key|Value
     *
     * @return string
     */
    public function getAttr($attrs)
    {
        $att = '';
        if (!empty($attrs)) {
            foreach ($attrs as $key => $value) {
                $att .= "{$key}='{$value}' ";
            }
        }

        return $att;
    }

    /**
     * Output Field.
     *
     * @since 1.0.0
     */
    public function field()
    {
        $output = '';
        $labelBefore = '';
        $labelAfter = '';
        $description = '';

        if (isset($this->args['label'])) {
            $labelBefore = sprintf('<label for="%s">%s', esc_attr($this->args['id']),
                esc_html__($this->args['label'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
            $labelAfter = '</label>';
        }

        if (isset($this->args['description'])) {
            $description = sprintf('<br><small>%s</small>',
                esc_html__($this->args['description'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
        }

        $output .= sprintf(
            '%1$s<input class="%2$s" placeholder="%3$s" type="%4$s" name="%5$s[%6$s]" id="%7$s" value="%8$s" %9$s/>%10$s %11$s',
            $labelBefore,
            esc_attr($this->args['class']),
            esc_attr($this->args['placeholder']),
            sanitize_key($this->args['type']),
            $this->page->optionsName,
            esc_attr($this->args['name']),
            esc_attr($this->args['id']),
            $this->escape(),
            $this->getAttr($this->args['attrs']),
            $labelAfter,
            $description
        );

        /**
         * Output Filter.
         *
         * @since 1.0.0
         *
         * @param string $output the output of the input type
         * @param Text The instance class
         */
        $output = apply_filters('wc_fatture_gratis_inv-input_text_output', $output, $this);

        /*
         * Before input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_text_before_input_action', $this->args);

        echo wp_kses($output, $this->allowHtml);

        /*
         * After input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_text_after_input_action', $this->args);
    }
}
