<?php

namespace WcFattureGratisInvoice\Admin\Settings\Fields;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Admin\Settings\OptionFields;
use WcFattureGratisInvoice\Admin\Settings\OptionPage;

class TextArea extends Text
{
    /**
     * Allow Html.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $allowHtml = [];

    /**
     * TextArea constructor.
     *
     * @since 1.0.0
     *
     * @param $args
     */
    public function __construct($args, OptionFields $fields, OptionPage $page)
    {
        $args = wp_parse_args($args, [
            'type' => 'textarea',
        ]);

        $this->allowHtml = [
            'label' => [
                'for' => true,
            ],
            'textarea' => [
                'class' => true,
                'placeholder' => true,
                'name' => true,
                'id' => true,
                'disabled' => true,
                'required' => true,
            ],
            'p' => true,
            'br' => true,
            'small' => true,
            'ul' => true,
            'li' => true,
            'strong' => true,
        ];

        parent::__construct($args, $fields, $page);
    }

    /**
     * Escape.
     *
     * @since  1.0.0
     *
     * @return string The escaped value of this type
     */
    public function escape()
    {
        $value = isset($this->fields->options[$this->args['name']]) ? $this->fields->options[$this->args['name']] : '';

        return wp_kses($value, [
            'p' => true,
            'br' => true,
            'small' => true,
            'ul' => true,
            'li' => true,
            'strong' => true,
        ]);
    }

    /**
     * Output Field.
     *
     * @since 1.0.0
     */
    public function field()
    {
        $output = '';
        $labelBefore = '';
        $labelAfter = '';
        $description = '';

        if (isset($this->args['label'])) {
            $labelBefore = sprintf('<label for="%s">%s', esc_attr($this->args['id']),
                esc_html__($this->args['label'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
            $labelAfter = '</label>';
        }

        if (isset($this->args['description'])) {
            $description = sprintf('<br><small>%s</small>',
                esc_html__($this->args['description'], WC_FATTUREGRATIS_INV_TEXTDOMAIN));
        }

        $output .= sprintf(
            '%1$s<textarea class="%2$s" placeholder="%3$s" name="%4$s[%5$s]" id="%6$s" %7$s>%8$s</textarea>%9$s %10$s',
            $labelBefore,
            esc_attr($this->args['class']),
            esc_attr($this->args['placeholder']),
            $this->page->optionsName,
            esc_attr($this->args['name']),
            esc_attr($this->args['id']),
            $this->getAttr($this->args['attrs']),
            $this->escape(),
            $labelAfter,
            $description
        );

        /**
         * Output Filter.
         *
         * @since 1.0.0
         *
         * @param string $output the output of the input type
         * @param TextArea The instance class
         */
        $output = apply_filters('wc_fatture_gratis_inv-input_textarea_output', $output, $this);

        /*
         * Before input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_textarea_before_input_action', $this->args);

        echo wp_kses($output, $this->allowHtml);

        /*
         * After input hook
         *
         * @since 1.0.0
         */
        do_action('wc_fatture_gratis_inv-input_textarea_after_input_action', $this->args);
    }
}
