<?php

namespace WcFattureGratisInvoice\Admin\Settings;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use WcFattureGratisInvoice\Plugin;

abstract class OptionFields
{
    /**
     * Holds the values to be used in the fields callbacks.
     *
     * @since 1.0.0
     *
     * @var array
     */
    public $options;

    /**
     * Contains the arguments for add fields settings.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $sectionArgs = [];

    /**
     * Contains the arguments for add fields settings.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $fieldsArgs = [];

    /**
     * Set section and fields.
     *
     * @since 1.0.0
     */
    protected function sectionFields()
    {
        include_once Plugin::getPluginDirPath('/inc/settings/pageSettingsFields.php');
    }

    /**
     * Add settings section and setting fields in an array.
     *
     * @since 1.0.0
     *
     * @return array|bool $settings_and_fields The settings section and setting fields in an array
     */
    public function sectionAndFieldsOptions()
    {
        $sectionsAndFields = [];

        // Settings section arguments
        if (!empty($this->sectionArgs)) {
            foreach ($this->sectionArgs as $key => $section) {
                $sectionsAndFields['section'][$key] = $section;
            }

            // Settings fields arguments
            $sectionsAndFields['fields'] = $this->fieldsArgs;
            if (empty($sectionsAndFields)) {
                return false;
            }
        }

        return $sectionsAndFields;
    }

    /*
     * Print the Section text settings
     *
     * @since 1.0.0
     */
    // public function sectionSettingsGeneralDescription()
    // {
    //     echo sprintf('<div class="wc_fatture_gratis_inv__description"><p>%1$s</p></div>',
    //         esc_html__('General options for enabling APIs',
    //             WC_FATTUREGRATIS_INV_TEXTDOMAIN)
    //     );
    // }

    /*
     * Print the Section text settings
     *
     * @since 1.0.0
     */
    // public function fieldsVatRules()
    // {
    //     $required    = esc_html__('required', WC_FATTUREGRATIS_INV_TEXTDOMAIN);
    //     $notRequired = esc_html__('not required', WC_FATTUREGRATIS_INV_TEXTDOMAIN);

    //     $output = '<table class="wc_fatture_gratis_inv-vat-rules wp-list-table widefat fixed striped">';
    //     $output .= '<thead>';
    //     $output .= '<tr>';
    //     $output .= '<td width="18%"><strong>' . esc_html__('Customer Country', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong></td>';
    //     $output .= '<td width="64%"><strong>' . esc_html__('The user selects: Company or Freelance', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong></td>';
    //     $output .= '<td width="18%"><strong>' . esc_html__('The user selects: Private', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong></td>';
    //     $output .= '<tr>';
    //     $output .= '</thead>';
    //     $output .= '<tbody>';
    //     $output .= '<tr>';
    //     $output .= '<td><strong>' . esc_html__('IT', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong></td>';
    //     $output .= '<td>' . sprintf(__('VAT number or Tax Code and E-mail (PEC) or the web-service code: %1$s',
    //             WC_FATTUREGRATIS_INV_TEXTDOMAIN),
    //             "<strong class='req'>{$required}</strong>"
    //         ) . '</td>';
    //     $output .= '<td>' . sprintf("<strong class='req'>%s %s</strong>", $required, esc_html__('(Tax Code)', WC_FATTUREGRATIS_INV_TEXTDOMAIN)) . '</td>';
    //     $output .= '</tr>';
    //     $output .= '<tr>';
    //     $output .= '<td><strong>' . esc_html__('UE', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong></td>';
    //     $output .= '<td>' . sprintf(__('VAT number or Tax Code: %1$s',
    //             WC_FATTUREGRATIS_INV_TEXTDOMAIN),
    //             "<strong class='not-req'>{$notRequired}</strong>"
    //         ) . '</td>';
    //     $output .= '<td>' . sprintf("<strong class='not-req'>%s</strong>", $notRequired) . '</td>';
    //     $output .= '</tr>';
    //     $output .= '<tr>';
    //     $output .= '<td><strong>' . esc_html__('Extra UE', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '</strong> ';
    //     $output .= '<small><abbr title="' . esc_html__('View (1)', WC_FATTUREGRATIS_INV_TEXTDOMAIN) . '">(1)</abbr></small></td>';
    //     $output .= '<td>' . sprintf(__('VAT number or Tax Code: %2$s',
    //             WC_FATTUREGRATIS_INV_TEXTDOMAIN),
    //             "<strong class='req'>{$required}</strong>",
    //             "<strong class='not-req'>{$notRequired}</strong>"
    //         ) . ' ';
    //     $output .= '</td>';
    //     $output .= '<td>' . sprintf("<strong class='not-req'>%s</strong>", $notRequired) . '</td>';
    //     $output .= '</tr>';
    //     $output .= '</tbody>';
    //     $output .= '</table>';

    //     echo $output;
    // }
}
