<?php

namespace WcFattureGratisInvoice\Functions;

use Exception;
use PSA\WS\FatturaPA\ArrayOfProdotti;
use PSA\WS\FatturaPA\ArrayOfScontoMaggiorazione;
use PSA\WS\FatturaPA\Destinatario;
use PSA\WS\FatturaPA\downloadFatturaPDF;
use PSA\WS\FatturaPA\Fattura;
use PSA\WS\FatturaPA\getXml;
use PSA\WS\FatturaPA\infoFattura;
use PSA\WS\FatturaPA\InvioFattureElettronicheService;
use PSA\WS\FatturaPA\Pagamento;
use PSA\WS\FatturaPA\Prodotti;
use PSA\WS\FatturaPA\ScontoMaggiorazione;
use PSA\WS\FatturaPA\sendInvoice;
use PSA\WS\FatturaPA\Sezionale;
use SoapFault;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

// use \WcFattureGratisInvoice\WooCommerce\Fields\GeneralFields;

/**
 * Send invoice to WebService.
 *
 * @since 1.0.0
 */
class WebService
{
    private $account;
    private $order;
    private $invio;
    private $options;
    private $GATEWAYS = [
        'cod' => 'MP01',
        'paypal' => 'MP08',
        'cheque' => 'MP02',
        'bacs' => 'MP05',
      ];

    public function __construct()
    {
        $this->options = \WcFattureGratisInvoice\Admin\Settings\OptionPage::init();
    }

    public function getOptions($modulo = 'fatturapa', $url = 'https://ws.fatturegratis.eu/index.php')
    {
        $this->hookActionLoadCertificate();
        $cert = 'cert';
        try {
            $context = stream_context_create([
                'http' => [
                    'user_agent' => 'PHPSoapClient',
                ],
                'ssl' => [
                    'crypto_method' => STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT,
                    'local_cert' => __DIR__.'/../../certs/'.$cert.'.pem',
                    'local_pk' => __DIR__.'/../../certs/'.$cert.'.key',
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                ],
            ]);

            $options = [
                'verifypeer' => false,
                'verifyhost' => false,
                'stream_context' => $context,
                'trace' => 1,
                'exceptions' => 1,
                'login' => null,
            ];

            $options['location'] = $url."?module=${modulo}";

            return $options;
        } catch (\Exception $e) {
            // echo $e->getMessage()."\n";
            return null;
        }
    }

    // public function hookActionPaymentConfirmation($orderId, $status_from, $status_to, $order)
    public function hookActionPaymentConfirmation($orderId)
    {
        $this->order = wc_get_order($orderId);
        // $this->order = $order;
        // echo $orderId;
        // echo $order->get_id();
        // \var_dump($this->order);
        // $this->order = new \WC_Order($orderId);
        if (false === $this->order) {
            return;
        }

        if ('completed' !== $this->order->get_status()) {
            return;
        }

        // No It return
        if ('IT' !== $this->order->get_billing_country()) {
            return;
        }
        $invoiceNumber = $this->order->get_meta('order_number_invoice');

        ini_set('soap.wsdl_cache_enabled', 0);
        ini_set('default_socket_timeout', 10);

        $options = $this->getOptions();

        $wsdl = 'https://ws.fatturegratis.eu/index.php?wsdl&module=fatturapa';

        update_post_meta($this->order->get_id(), '_invoice_ws_status', 'undefined');
        try {
            $this->invio = new InvioFattureElettronicheService($options, $wsdl);
        } catch (SoapFault $sf) {
            // throw new HttpException(500, 'Fail to connect with WebService.');
            // echo "catch";
            $this->invio = null;
        }
        $block = false;

        if ($this->invio and !$block) {
            $uuid = $this->order->get_meta('_invoice_ws_uuid');

            $invoice = $this->create_invoice();
            if (!$uuid and !$block) {
                update_post_meta($this->order->get_id(), '_invoice_ws_status', 'sending');
                try {
                    $param_sendInvoice = new sendInvoice($invoice);
                    $id_ade = $this->invio->sendInvoice($param_sendInvoice);

                    $uuid = $id_ade->getSendInvoiceResult();

                    update_post_meta($this->order->get_id(), '_invoice_ws_uuid', $uuid);
                } catch (Exception $error) {
                    update_post_meta($this->order->get_id(), '_invoice_ws_status', 'send failed');
                }
            }
            if ($uuid) {
                $status = $this->checkInfo($uuid);
                update_post_meta($this->order->get_id(), '_invoice_ws_status', $status);
                if ('success' == $status) {
                    $this->downloadPdf($uuid);
                    $this->downloadXml($uuid);
                }
            }
        }
    }

    public function hookActionLoadCertificate()
    {
        //TODO find a way to call this hook on save data button

        $target_dir = __DIR__.'/../../certs/';
        $cert_key = $target_dir.'cert.key';
        $cert_pem = $target_dir.'cert.pem';
        $cert_info = [];
        $file = get_attached_file($this->options->getOptions('invoice_cert_file_id'));
        $password = $this->options->getOptions('password_certificate');

        if (file_exists($file) and '' != $password) {
            if (!$cert_store = file_get_contents($file)) {
                echo "Error: Reupload your certificate file\n";
                exit;
            }

            if (!openssl_pkcs12_read($cert_store, $cert_info, $password)) {
                echo "Error: Verify your certificate password.\n";
                print_r($cert_info);
                exit;
            }
            $new_password = null;
            $result = null;

            $worked = openssl_pkey_export($cert_info['pkey'], $result, $new_password);
            if ($worked) {
                file_put_contents($target_dir.'cert.key', $result);
            } else {
                echo openssl_error_string();
            }

            $worked = openssl_x509_export($cert_info['cert'], $result);
            if ($worked) {
                file_put_contents($target_dir.'cert.pem', $result);
            } else {
                echo openssl_error_string();
            }
            $this->options->setOption('password_certificate', '');
            unlink($file);
        }
    }

    public function checkInfo(string $uuid): string
    {
        if ($this->invio) {
            try {
                $param = new infoFattura($uuid);
                $d_info = $this->invio->infoFattura($param);
                if (!$d_info) {
                    return 'error';
                }
                $info = $d_info->getInfoFatturaResult();
                //TODO NISSUNA RIZULTANZA
                // return 'processing' (NISSUNA RIZU);
                // echo "<h3> Check INFO </h3> </br>";
                // var_dump($info);
                // echo "<h5>info</h5>";
                $myfile = fopen(__DIR__.'/../../files/info/'.$uuid.'.info', 'w') or exit('Unable to open file!');
                fwrite($myfile, $info);
                $json = json_decode($info, false);
                // echo "<h3> Check error </h3> </br>";
                // var_dump($json);
            } catch (Exception $e) {
                // echo "exception";
                // var_dump($e);
                return 'error';
            }
        } else {
            return 'error';
        }

        return 'success';
    }

    //--------------------------------------------------------------------------
    public function downloadXml($uuid)
    {
        $doc_file = __DIR__.'/../../files/xml/'.$uuid.'.xml';
        if (file_exists($doc_file)) {
            // echo "<h3>file xml".$uuid." exists</h3><br/>";
            // unlink($doc_file);
            return;
        }

        $param = new getXml($uuid);
        while (1) {
            $xml = $this->invio->getXml($param);
            if (null != $xml && null != $xml->getGetXmlResult() && '' != $xml->getGetXmlResult()) {
                $doc = base64_decode($xml->getGetXmlResult());
                $myfile = fopen($doc_file, 'w') or exit('Unable to open file!');
                fwrite($myfile, $doc);
                break;
            }
            sleep(1);
        }
    }

    public function downloadPdf($uuid)
    {
        $param = new downloadFatturaPDF($uuid);
        $doc_file = __DIR__.'/../../files/pdf/'.$uuid.'.pdf';
        if (file_exists($doc_file)) {
            // echo "<h3>file pdf ".$uuid." exists</h3><br/>";
            // unlink($doc_file);
            return;
        }

        for ($x = 0; $x < 60; ++$x) {
            $pdf = $this->invio->downloadFatturaPDF($param);
            if (null != $pdf && null != $pdf->getDownloadFatturaPDFResult() && '' != $pdf->getDownloadFatturaPDFResult()) {
                $doc = base64_decode($pdf->getDownloadFatturaPDFResult());
                // echo "<h5> doc pdf </h5>";
                $myfile = fopen($doc_file, 'w') or exit('Unable to open file!');
                fwrite($myfile, $doc);

                break;
            }
            sleep(2);
        }
    }

    public function create_invoice()
    {
        // return;
        // TODO: Change parameters for retrieve by configuration
        // __construct($auto_send, $bollo_virtuale, $invia_proforma, $data_documento, $data_ricorrenza, $intervallo_ricorrenza, $numero_ricorrenze, $forza_firma, $gestisci_minimi)
        $f = new Fattura(
            true,
            false,
            true,
            $this->order->get_date_created() ? $this->order->get_date_created()->getTimestamp() : null,
            0,
            0,
            0,
            false,
            false
        );

        if (get_option('wc_fatture_gratis_inv-settings-section')) {
            $section = $this->create_section(get_option('wc_fatture_gratis_inv-settings-section'));
            $f->setSezionale($section);
        }
        // $category = $this->create_category($invoice->getCategory());
        // $f->setCategoria($category);
        // $f->setCategoria($invoice->getCategory());

        $PA = ['company_participated', 'public_administration'];
        $paOrPr = 'PR';
        if (in_array($this->order->get_meta('_billing_invoice_type'), $PA)) {
            $paOrPr = 'PA';
        }

        // $f->setNumero_fattura($this->order->get_meta('order_number_invoice'));
        // $f->setRegime_fiscale(GeneralFields::getGeneralInvoiceOptionTaxRegime());
        $f->setTipo_documento($this->order->get_meta('_billing_choice_type'));
        // $f->setTipo_documento($this->order->get_meta('td01'));
        $f->setDivisa($this->order->get_currency());
        $f->setPa_or_pr($paOrPr);
        // $f->setOggetto($invoice->getSubject());
        // $f->setCondizioni($invoice->getConditions());
        $f->setSoggetto_emittente('TZ');

        // $mittente = $this->create_sender();
        // $f->setMittente($mittente);

        $destinatario = $this->create_receiver();
        $f->setDestinatario($destinatario);

        // var_dump($this->order->get_taxes( ));
        $vat = 0.00;
        foreach ($this->order->get_taxes() as $key => $value) {
            $tax = \WC_Tax::_get_tax_rate($value->get_rate_id());
            if (in_array(strtolower($tax['tax_rate_name']), ['iva', 'vat'])) {
                $vat = floatval($tax['tax_rate']);
            }
        }

        $prodotti = [];
        foreach ($this->order->get_items() as $key => $value) {
            $prodotti[] = $this->create_product($value, $vat);
        }
        $products = new ArrayOfProdotti();
        $products->setItem($prodotti);
        $f->setProdotti($products);

        $sm = new ArrayOfScontoMaggiorazione();

        $totalDiscount = $this->order->get_total_discount();
        $totalShipping = $this->order->get_shipping_total();
        $array_discounts = [];
        if ($totalDiscount) {
            $array_discounts[] = $this->create_discount('SC', floatval($totalDiscount));
        }
        if ($totalShipping) {
            $array_discounts[] = $this->create_discount('MG', floatval($totalShipping));
        }

        $sm->setItem($array_discounts);
        $f->setScontoMaggiorazione($sm);

        // $f->setPercentuale_ritenuta($invoice->getPercentageWithholding());
        // $f->setCausale_ritenuta($invoice->getCausalWithholding());

        $pagamento = $this->create_payment();
        $f->setPagamento($pagamento);

        // if ($invoice->getPayment() && $invoice->getHasForecastingBank() && $invoice->getPayment()->getBeneficiary()->getForecastingBank()) {
        //     $cassa = $this->create_forecasting_bank($invoice->getPayment()->getBeneficiary()->getForecastingBank());
        //     $f->setCassaPrevidenziale($cassa);
        // }

        // $files = $invoice->getAttachments();
        //
        // if (!is_null($files) && !$ingore_files) {
        //     $attachments = [];
        //     foreach ($files as $media_object) {
        //         // $file = new File($this->uploadDirectory.'/'.$file_name);
        //         $file = $media_object->file;
        //         array_push($attachments, $this->read_attachment($file));
        //     }
        //
        //     $array_allegato = new \ArrayOfAllegato();
        //     $array_allegato->setItem($attachments);
        //     $f->setAllegati($array_allegato);
        // }

        // TODO: Implement
        // $f->setOrdine_acquisto();
        // $f->setContratto();
        // $f->setConvenzione();
        // $f->setRicezione();
        // $f->setFatture_correlate();
        // echo "<h5>invoice</h5>";
        // $myfile = fopen("/home/juliano/debug-logs/invoice.log", "w") or die("Unable to open file!");
        // fwrite($myfile, print_r($f, true));
        return $f;
    }

    private function create_section(string $section)
    {
        $entity = new Sezionale();
        $entity->setSezionale($section);
        $entity->setSuffisso($section);

        return $entity;
    }

    private function create_receiver()
    {
        // The main address pieces:
        $address = $this->order->get_billing_address_1();
        $address_2 = $this->order->get_billing_address_2();
        $postcode = $this->order->get_billing_postcode();

        $country = $this->order->get_billing_country();
        $state = $this->order->get_billing_state();
        $city = $this->order->get_billing_city();

        $physicalPersonOptions = ['freelance', 'private'];
        $physicalPerson = in_array($this->order->get_meta('_billing_invoice_type'), $physicalPersonOptions);

        $entity = new Destinatario();
        $entity->setCodice_destinatario($this->order->get_meta('_billing_recipient_code'));
        $entity->setRagione_sociale($this->order->get_billing_company());
        $entity->setTitolo('');
        $entity->setNome($this->order->get_billing_first_name());
        $entity->setCognome($this->order->get_billing_last_name());
        $entity->setPiva($this->order->get_meta('_billing_vat_number'));
        $entity->setCodice_fiscale($this->order->get_meta('_billing_tax_code'));
        $entity->setIndirizzo(sprintf('%s - %s', $address, $address_2));
        $entity->setCap($postcode);
        $entity->setCitta($city);
        $entity->setProvincia($state);
        $entity->setNazione($country);
        $entity->setEmail($this->order->get_meta('_billing_pec') ?: $this->order->get_billing_email());
        $entity->setPersona_fisica($physicalPerson);

        return $entity;
    }

    private function create_product(\WC_Order_Item $invoice_product, float $vat = 0.00)
    {
        if ('taxable' != $invoice_product->get_tax_status()) {
            $vat = 0.00;
        }

        $product = $invoice_product->get_product();
        $quantity = $invoice_product->get_quantity();
        // $discounts = $invoice_product->getDiscounts();

        // $entity = new \Prodotti($product->getWithholding());

        $entity = new Prodotti(null);
        $entity->setProdotto($product->get_name());
        $entity->setDescrizione($product->get_description());
        $entity->setIva($vat);
        $entity->setQuantita($quantity);
        $entity->setPrezzo(floatval($product->get_price()));
        // $entity->setUnita($product->getUnit()->getName());
        // $entity->setRitenuta($product->getWithholding());
        // $entity->setEsigibilitaIVA($vatPayment);
        // $entity->setNatura($product->getVatNature());

        return $entity;
    }

    private function create_discount(string $type, float $value)
    {
        $sconto = new ScontoMaggiorazione();
        $sconto->setTipo($type);
        $sconto->setSconto($value);

        return $sconto;
    }

    private function create_discounts(array $discounts)
    {
        $array_discounts = [];
        foreach ($discounts as $key => $value) {
            $array_discounts[] = $this->create_discount('SC', $value['value']);
        }

        return $array_discounts;
    }

    private function create_payment()
    {
        $gateway = $this->order->get_payment_method();

        if (isset($this->GATEWAYS[$gateway])) {
            $paymentCode = $this->GATEWAYS[$gateway];
        } else {
            $paymentCode = 'MP08';
        }

        // $beneficiary = get_option('general_store_company_name') || sprintf('%s %s', get_option('general_store_your_name'), get_option('general_store_your_surname'));
        $entity = new Pagamento();
        // $entity->setCondizioniPagamento($payment->getConditions());
        // $entity->setBeneficiario($beneficiary);
        $entity->setModalitaPagamento($paymentCode);
        $entity->setDataScadenzaPagamento($this->order->get_date_paid()->format('Y-m-d'));
        $entity->getImportoPagamento($this->order->get_total());
        // $entity->setIBAN('');

        return $entity;
    }
}
