<?php

namespace WcFattureGratisInvoice;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

final class Plugin
{
    /**
     * Sanitize path.
     *
     * @since 1.0.0
     *
     * @param string $path The path to sanitize
     *
     * @return string the sanitized path
     */
    private static function sanitizePath($path)
    {
        while (false !== strpos($path, '..')) {
            $path = str_replace('..', '', $path);
        }

        $path = ('/' !== $path) ? $path : '';

        return $path;
    }

    /**
     * Get Plugin Dir Path.
     *
     * @since  1.0.0
     *
     * @param string $path     The path to append to the plugin dir path. Optional. Default to '/'.
     * @param bool   $override If override from theme true o false
     *
     * @return string The plugin dir path
     */
    public static function getPluginDirPath($path = '/', $override = false)
    {
        $path = untrailingslashit(plugin_dir_path(__DIR__)).'/'.trim($path, '/');
        $located = realpath($path);

        $locatedTheme = get_theme_file_path($path);

        if (file_exists($locatedTheme) && $override) {
            $located = $locatedTheme;
        }

        return self::sanitizePath($located);
    }

    /**
     * Get Plugin Dir Url.
     *
     * @since  1.0.0
     *
     * @param string $path     The path to append to the plugin dir url. Optional. Default to '/'.
     * @param bool   $override If override from theme true o false
     *
     * @return string The plugin dir url
     */
    public static function getPluginDirUrl($path = '/', $override = false)
    {
        $located = untrailingslashit(plugin_dir_url(__DIR__)).'/'.trim($path, '/');

        $locatedTheme = get_theme_file_path($path);

        if (file_exists($locatedTheme) && $override) {
            $located = $locatedTheme;
        }

        return self::sanitizePath($located);
    }
}
