<?php

namespace WcFattureGratisInvoice;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

final class Resources
{
    /**
     * Register.
     *
     * @since 1.0.0
     */
    public function register()
    {
        // Styles
        $styles = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/styles.php');

        if (is_array($styles) && !empty($styles)) {
            foreach ($styles as $style) {
                // Register.
                if (isset($style['register']) && true === $style['register']) {
                    wp_register_style(
                        $style['handle'],
                        $style['file'],
                        $style['deps'],
                        $style['ver'],
                        $style['media']
                    );
                }
            }
        }

        // Scripts
        $scripts = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/scripts.php');

        if (is_array($scripts) && !empty($scripts)) {
            foreach ($scripts as $script) {
                // Register.
                if (isset($script['register']) && true === $script['register']) {
                    wp_register_script(
                        $script['handle'],
                        $script['file'],
                        $script['deps'],
                        $script['ver'],
                        $script['in_footer']
                    );
                }
            }
        }
    }

    /**
     * Enqueue.
     *
     * @since 1.0.0
     */
    public function enqueue()
    {
        // Styles
        $styles = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/styles.php');

        if (is_array($styles) && !empty($styles)) {
            foreach ($styles as $style) {
                if (isset($style['enqueue']) && true === $style['enqueue']) {
                    wp_enqueue_style(
                        $style['handle'],
                        $style['file'],
                        $style['deps'],
                        $style['ver'],
                        $style['media']
                    );
                } elseif (!isset($style['enqueue']) && !isset($style['register'])) {
                    wp_enqueue_style(
                        $style['handle'],
                        $style['file'],
                        $style['deps'],
                        $style['ver'],
                        $style['media']
                    );
                }
            }
        }

        // Scripts
        $scripts = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/scripts.php');

        if (is_array($scripts) && !empty($scripts)) {
            foreach ($scripts as $script) {
                if (isset($script['enqueue']) && true === $script['enqueue']) {
                    wp_enqueue_script(
                        $script['handle'],
                        $script['file'],
                        $script['deps'],
                        $script['ver'],
                        $script['in_footer']
                    );
                } elseif (!isset($script['enqueue']) && !isset($script['register'])) {
                    wp_enqueue_script(
                        $script['handle'],
                        $script['file'],
                        $script['deps'],
                        $script['ver'],
                        $script['in_footer']
                    );
                }
            }
        }

        do_action('wc_fatture_gratis_inv-after_admin_enqueue_scripts');
    }

    /**
     * Localize Scripts.
     *
     * @since 1.0.0
     */
    public function localizeScript()
    {
        // Localized Scripts
        $scripts = include \WcFattureGratisInvoice\Plugin::getPluginDirPath('/inc/localizeScripts.php');

        if (is_array($scripts) && !empty($scripts)) {
            foreach ($scripts as $handle => $data) {
                if (wp_script_is($handle, 'registered') && wp_script_is($handle, 'enqueued')) {
                    wp_localize_script(
                        $handle,
                        $handle,
                        $data
                    );
                }
            }
        }
    }

    /**
     * Load WordPress Media Library.
     *
     * @since 1.0.0
     */
    public function loadWpMediaFiles($page)
    {
        // only to page of plugin
        if ('toplevel_page_wc_fatture_gratis_inv-options-page' == $page) {
            // Enqueue WordPress media scripts
            wp_enqueue_media();
            // Enqueue custom script that will interact with wp.media
            wp_enqueue_script('fatture_gratis_inv_script', plugins_url('/../assets/js/select_media.js', __FILE__), ['jquery'], '0.1');
        }
    }
}
