<?php

namespace WcFattureGratisInvoice\Utils;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class TimeZone
{
    /**
     * Time Zone.
     *
     * @since  1.0.0
     *
     * @var \DateTimeZone The date time zone based on option
     */
    private $timeZone;

    /**
     * Get the TimeZone.
     *
     * Retrieve the timezone based on WordPress option.
     *
     * @since  1.0.0
     *
     * @return string The timezone option value
     */
    private function getTimeZoneOption()
    {
        // Timezone_string is empty when the option is set to Manual Offset. So we use gmt_offset.
        $option = get_option('timezone_string') ? get_option('timezone_string') : get_option('gmt_offset');
        // Set to UTC in order to prevent issue if used with DateTimeZone constructor.
        $option = (in_array($option, ['', '0'], true) ? 'UTC' : $option);
        // And remember to add the symbol.
        if (is_numeric($option) && 0 < $option) {
            $option = '+'.$option;
        }

        return $option;
    }

    /**
     * Create the time zone instance.
     *
     * @since  1.0.0
     *
     * @return \DateTimeZone
     */
    private function createTimeZone()
    {
        // Get the option.
        $option = $this->getTimeZoneOption();

        // Return the new instance.
        return new \DateTimeZone($option);
    }

    /**
     * TimeZoneOption constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        // Set the time zone.
        $this->timeZone = $this->createTimeZone();
    }

    /**
     * Get the Time Zone.
     *
     * @since  1.0.0
     *
     * @return \DateTimeZone The \DateTimeZone instance
     */
    public function getTimeZone()
    {
        return $this->timeZone;
    }
}
