# Fatture Gratis WS Client

Client for connect to Web Service

------

> Example of code to generate authentication options: 
```php
<?php

/**
 * WebServiceOptions.php
 * Generate array of options to connect with Web Service
 * PHP version 8.0.
 *
 * @category  PSA
 *
 * @author    Diefferson Koderer Môro <contact@djpremier.dev>
 * @copyright 2019 Phoenix ICT
 * @license   PSA Phoenix ICT <https://psa.phoenixict.it/license/>
 *
 * @see       https://psa.phoenixict.it/
 */

declare(strict_types=1);

namespace App\Service\FattureGratis;

use ErrorException;
use Exception;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use stdClass;
use Throwable;

// Run `composer require psr/log` to install logger
class WebServiceOptions implements LoggerAwareInterface
{
    use LoggerAwareTrait;

    public function __construct(
        protected string $webServiceUrl,
        protected string $webServiceCertificate,
        protected string $webServiceSecretKey,
        protected string $webServiceCertificateAuthority
    ) {
    }

    public function getOptions(
        ?string $user = null,
        ?string $token = null,
        string $module = 'fatturapa'
    ): ?array {
        try {
            $authJson = null;

            if ($user && !$token) {
                throw new Exception(sprintf("Trying to perform an activity with user '%s' without specifying the token", $user), 403);
            }

            if ($token) {
                $auth = new stdClass();
                $auth->uuid_utente = $user;
                $auth->token = $token;
                $authJson = base64_encode(json_encode($auth));

                if (empty($authJson)) {
                    throw new Exception('There was an error creating the authorization token', 400);
                }
            }

            // https://www.php.net/manual/pt_BR/context.ssl.php
            $context = stream_context_create(
                [
                    'http' => [
                        'user_agent' => 'PHPSoapClient',
                    ],
                    'ssl' => [
                        'verify_peer' => false,
                        'verify_peer_name' => false,
                        'cafile' => $this->webServiceCertificateAuthority,
                        'local_cert' => $this->webServiceCertificate,
                        'local_pk' => $this->webServiceSecretKey,
                    ],
                ]
            );

            // https://www.php.net/manual/pt_BR/soapclient.construct.php
            $options = [
                'trace' => 1,
                'exceptions' => 1,
                'cache_wsdl' => 'WSDL_CACHE_NONE',
                'user_agent' => 'PHPSoapClient',
                'stream_context' => $context,
                'login' => $authJson,
                'location' => sprintf('%s?module=%s', $this->webServiceUrl, $module),
            ];

            return $options;
        } catch (Exception | ErrorException | Throwable $ex) {
            $this->logger->error(
                sprintf(
                    'Fail getOptions WebService: %s',
                    strtoupper($module)
                ),
                [
                    'exception' => (string) $ex,
                    'user' => $user,
                    'token' => $token,
                    'module' => $module,
                ]
            );
            throw new Exception('Fail to connect with WebService', 504);
        }
    }
}
```